import numpy as np
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import pystan

#
#           データの読み込み
#
flnm = input('Data file name = ')
fin = open(flnm, 'r')
data = fin.readlines()
fin.close()
pos = 0
while True:
    if len(data[pos]) > 0:
        if data[pos][0] == '/':
            break
    pos += 1

RTs = []
Intvls = []
pos += 2
while True:
    if data[pos][0] == '/':
        break
    slist = data[pos].split()
    RTs.append(float(slist[0]))
    Intvls.append(float(slist[1]))
    pos += 1

for x, y in zip(RTs, Intvls):
    print(x, '   ', y)

RTs = np.array(RTs)

xi_hat = RTs.min()          #   シフト値の最大値
print('xi_hat = ', xi_hat)
RTs_xi = RTs - xi_hat
#
#       モーメント法に依るパラメータ値の推定
#
beta_hat = RTs_xi.mean() / RTs_xi.var()
print('beta_hat = ', beta_hat)
alpha_hat = beta_hat * RTs_xi.mean()
print('alpha_hat = ', alpha_hat)

max_a = alpha_hat * 100.0
max_b = beta_hat * 100.0
max_x = xi_hat
Data = {'N': len(RTs), 'RT': RTs, 'max_a': max_a, 'max_b': max_b, 'max_x': max_x}

sm = pystan.StanModel(file = 'shiftedGamma.stan')   #   ガンマ分布
fit = sm.sampling(data = Data, n_jobs = 1)

param_a = fit['param_a']
param_b = fit['param_b']
param_x = fit['param_x']
plt.hist(param_a)
plt.xlabel(r'$\alpha$', fontsize = 14)
plt.title(r'len($\alpha$) = {}'.format(len(param_a)), fontsize = 18)
plt.show()

plt.hist(param_b)
plt.xlabel(r'$\beta$', fontsize = 14)
plt.title(r'len($\beta$) = {}'.format(len(param_b)), fontsize = 18)
plt.show()

plt.hist(param_x)
plt.xlabel(r'$\xi$', fontsize = 14)
plt.title(r'len($\xi$) = {}'.format(len(param_x)), fontsize = 18)
plt.show()

#
#       中央値
#
med_a = np.percentile(param_a, 50)
med_b = np.percentile(param_b, 50)
med_xi = np.percentile(param_x, 50)
print('med_a = ', med_a, '   med_b = ', med_b, '   med_xi = ', med_xi)

max_sgm = (RTs.var()**0.5) * 10.0
min_mu = RTs.mean() - max_sgm
max_mu = RTs.mean() + max_sgm

Data = {'N': len(RTs), 'RT': RTs, 'min_mu': min_mu, 'max_mu': max_mu, 'max_sgm': max_sgm}
sm1 = pystan.StanModel(file = 'Normal.stan')    #   正規分布
fit1 = sm1.sampling(data = Data, n_jobs = 1)

param_mu = fit1['mu']
param_sgm = fit1['sgm']
plt.hist(param_mu)
plt.xlabel('$\mu$', fontsize = 14)
plt.title(r'len($\mu$) = {}'.format(len(param_mu)))
plt.show()

plt.hist(param_sgm)
plt.xlabel('$\sigma$', fontsize = 14)
plt.title(r'len($\sigma$) = {}'.format(len(param_sgm)))
plt.show()

#
#       中央値
#
med_mu = np.percentile(param_mu, 50)
med_sgm = np.percentile(param_sgm, 50)
print('med_mu = ', med_mu, '   med_sgm = ', med_sgm)

rt = np.arange(0, 0.5, 0.001)
#
#       ガンマ分布
#
pdf_values = ss.gamma.pdf(rt, med_a, med_xi, 1/med_b)
plt.plot(rt, pdf_values, linewidth = 3, label = 'gamma(ξ,α,β)')
#
#       正規分布
#
pdf1_values = ss.norm.pdf(rt, med_mu, med_sgm)
plt.plot(rt, pdf1_values, linewidth = 3, label = 'normal(μ,σ)')
#
#       データ
#
plt.hist(RTs, density = True, label = 'RT')

plt.xlabel('Time', fontsize = 14)
plt.title('ξ = {0:<.3f}, α = {1:<.3f}, β = {2:<.3f}\n'.format(med_xi, med_a, med_b) +\
          'μ = {0:<.3f}, σ = {1:<.3f}'.format(med_mu, med_sgm),
          fontsize = 16)
plt.legend(fontsize = 14)
plt.show()
